// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <map>
#include <string>
#include <fstream>

#include <iup.h>
#include <iupgl.h>
#include <iupcontrols.h>

#include "scene.h"
#include "util.h"

using std::map;
using std::string;
using std::ifstream;

static string exec_path;

static Ihandle* canvas = NULL;

static SceneGraph::cScene* SG = NULL;
static map<string, SceneGraph::cAppearance*> appList;
static map<string, SceneGraph::cTexture*> texList;
static map<string, SceneGraph::cShape*> shapeList;

static float left_winWidth = 0.0;
static float left_winHeight = 0.0;

static float hTable = 0.0;

static float dxTable = 0.0;
static float dyTable = 0.0;
static float dzTable = 0.0;

static float dxFoot = 0.0;
static float dyFoot = hTable;
static float dzFoot = 0.0;

static float radius = 5.0;

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetExecPath(char* s)
{
 exec_path = DropFilename((string)s);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int cbExit(void)
{
 if(SG)
  delete SG;

 appList.clear();
 texList.clear();
 shapeList.clear();

 return IUP_CLOSE;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbOpenTriangles(Ihandle*)
{
/* if(bunny)
 {
  delete bunny;
  bunny = NULL;
 }
 if(bunny2)
 {
  delete bunny2;
  bunny2 = NULL;
 }

 string filename;
 if(FileDialog(IUP_OPEN, "Open File", "*.msh|*.msh", filename))
 {
  std::ifstream f;
  f.clear();
  f.open(filename.c_str(), std::ios::in);
  if(f)
  {
   PopupGauge();
   Ihandle* gauge = IupGetHandle("gauge");
   bunny = new cObject(f, gauge, true);
   bunny2= new cObject(*bunny, true);
   f.close();
   KillGauge();
  }
 }
 
 IupRefresh(canvas);
 IupRefresh(canvas2);
 */
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbOpenTetrahedrons(Ihandle*)
{
/* if(bunny)
 {
  delete bunny;
  bunny = NULL;
 }
 if(bunny2)
 {
  delete bunny2;
  bunny2 = NULL;
 }

 string filename;
 if(FileDialog(IUP_OPEN, "Open File", "*.msh|*.msh", filename))
 {
  std::ifstream f;
  f.clear();
  f.open(filename.c_str(), std::ios::in);
  if(f)
  {
   PopupGauge();
   Ihandle* gauge = IupGetHandle("gauge");
   bunny = new cObject(f, gauge, false);
   bunny2= new cObject(*bunny, false);
   f.close();
   KillGauge();
  }
 }

 IupRefresh(canvas);
 IupRefresh(canvas2);
 */
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createCamera(void)
{
 //camera
 cCoord cam_pos, cam_center, cam_up;
 
 cam_pos.x = 0.0;
 cam_pos.y = 0.0;
 cam_pos.z = 0.0;

 cam_center.x = 0.0;
 cam_center.y = -1.0;
 cam_center.z = 0.0;

 cam_up.x = 0.0;
 cam_up.y = 0.0;
 cam_up.z = 1.0;

 SceneGraph::cCamera* cam = new SceneGraph::cCamera("camera1", (GLfloat)60.0, (GLint)left_winWidth/(GLint)left_winHeight,
                                                    (GLfloat)0.1, (GLfloat)10000.0, cam_pos, cam_center, cam_up);

 //camera transform
 SceneGraph::cTransform* camTransf = new SceneGraph::cTransform("camTransf");
 camTransf->addChild(cam->label(), (SceneGraph::cNode*)cam);

 //camera aim
 camTransf->Translate(0.0, 200.0, 0.0);
 //camTransf->Rotate(20.0, 1.0, 0.0, 0.0);
 

 //add in scene
 SG->addChild(camTransf->label(), (SceneGraph::cNode*)camTransf);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createLight(void)
{
 //light
 float light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
 float light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
 float light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
 float light_position[] = { 100.0, 100.0, 100.0, 1.0 };
 SceneGraph::cLight* light = new SceneGraph::cLight("light1", light_position, light_ambient, light_diffuse, light_specular);

 SG->addChild(light->label(), (SceneGraph::cNode*)light);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createTableMaterial(void)
{
 //material
 float mat_ambient[] = { 0.18, 0.10, 0.0, 1.0 };
 float mat_diffuse[] = { 0.36, 0.18, 0.0, 1.0 };
 float mat_specular[] = { 0.36, 0.18, 0.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["tableMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createSphereMaterial(void)
{
 //material
 float mat_ambient[] = { 0.05, 0.2, 0.2, 1.0 };
 float mat_diffuse[] = { 0.08, 0.46, 0.58, 0.7 };
 float mat_specular[] = { 0.08, 0.46, 0.58, 0.7 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["sphereMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createBlueCubeMaterial(void)
{
 //material
 float mat_ambient[] = { 0.0, 0.0, 0.2, 1.0 };
 float mat_diffuse[] = { 0.0, 0.0, 1.0, 1.0 };
 float mat_specular[] = { 0.0, 0.0, 1.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["blueCubeMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createYellowCubeMaterial(void)
{
 //material
 float mat_ambient[] = { 0.2, 0.2, 0.0, 1.0 };
 float mat_diffuse[] = { 1.0, 1.0, 0.0, 1.0 };
 float mat_specular[] = { 1.0, 1.0, 0.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["yellowCubeMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createRedCubeMaterial(void)
{
 //material
 float mat_ambient[] = { 0.2, 0.0, 0.0, 1.0 };
 float mat_diffuse[] = { 1.0, 0.0, 0.0, 1.0 };
 float mat_specular[] = { 1.0, 0.0, 0.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["redCubeMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createDadoMaterial(void)
{
 //material
 float mat_ambient[] = { 0.2, 0.2, 0.0, 1.0 };
 float mat_diffuse[] = { 0.9, 0.9, 0.7, 1.0 };
 float mat_specular[] = { 0.9, 0.9, 0.7, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["dado"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createBunnyMaterial(void)
{
 //material
 float mat_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
 float mat_diffuse[] = { 0.68, 0.67, 0.22, 1.0 };
 float mat_specular[] = { 0.68, 0.67, 0.22, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["bunnyMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createFloorMaterial(void)
{
 //material
 float mat_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
 float mat_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
 float mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["floorMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createDesktopMaterial(void)
{
 //desktop material
 float mat_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
 float mat_diffuse[] = { 0.0, 0.0, 0.0, 1.0 };
 float mat_specular[] = { 0.0, 0.0, 0.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["desktopMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createPaperMaterial(void)
{
 //desktop material
 float mat_ambient[] = { 1.0, 1.0, 1.0, 1.0 };
 float mat_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
 float mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
 float mat_shininess = 8.0;
 SceneGraph::cMaterial* material = new SceneGraph::cMaterial(mat_ambient, mat_diffuse, mat_specular, mat_shininess);
 appList["paperMaterial"] = (SceneGraph::cAppearance*)material;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createMaterials(void)
{
 createTableMaterial();
 createSphereMaterial();
 createBlueCubeMaterial();
 createYellowCubeMaterial();
 createRedCubeMaterial();
 createBunnyMaterial();
 createFloorMaterial();
 createDesktopMaterial();
 createPaperMaterial();
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createTexture(void)
{
 string filename = exec_path + "database\\wood.bmp";
 SceneGraph::cTexture* texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["wood"] = texture;

 filename = exec_path + "database\\wood1.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["wood1"] = texture;

 filename = exec_path + "database\\dado1.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["dado1"] = texture;

 filename = exec_path + "database\\dado3.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["dado3"] = texture;

 filename = exec_path + "database\\dado6.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["dado6"] = texture;

 filename = exec_path + "database\\checker.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["checker"] = texture;

 filename = exec_path + "database\\areia2.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["wall"] = texture;

 filename = exec_path + "database\\picture.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["picture"] = texture;

 filename = exec_path + "database\\window.bmp";
 texture = NULL;
 texture = new SceneGraph::cTexture(filename);
 texList["window"] = texture;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createCubes(void)
{
 float dim = 3.0;

 /* BLUE CUBE */

 //shape
 SceneGraph::cRect* cube = new SceneGraph::cRect(dim, dim, dim);
 shapeList["cube"] = (SceneGraph::cShape*)cube;

 //material
 SceneGraph::cAppearance* material = NULL;
 material = (SceneGraph::cAppearance*)appList["dado"];

 //entity
 SceneGraph::cEntity* entity = new SceneGraph::cEntity("blueCube", (SceneGraph::cShape*)cube, material);

 //texture
 SceneGraph::cTexture* dado = NULL;
 dado = texList["dado6"];
 entity->SetTexture(dado);

 //transform
 SceneGraph::cTransform* cubeTransf = new SceneGraph::cTransform("cubeTransf");
 cubeTransf->Translate(0.5*(dxTable/2), (dyTable/2)+(dim/2), -0.5*(dzTable/2)); 
 cubeTransf->addChild(entity->label(), (SceneGraph::cNode*)entity);

 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");

 SceneGraph::cTransform* topTransf = NULL;
 if(tableTransf)
  topTransf = (SceneGraph::cTransform*)tableTransf->getChild("topTransf");

 if(topTransf)
  topTransf->addChild(cubeTransf->label(), (SceneGraph::cNode*)cubeTransf);

 
 /* YELLOW CUBE */

 //entity
 entity = new SceneGraph::cEntity("yellowCube", (SceneGraph::cShape*)cube, material);

 //texture
 dado = NULL;
 dado = texList["dado1"];
 entity->SetTexture(dado);

 //transform
 SceneGraph::cTransform* yellowCubeTransf = new SceneGraph::cTransform("yellowCubeTransf");
 yellowCubeTransf->Translate(0.0, 0.0, -1.2*dim); 
 yellowCubeTransf->Rotate(45.0, 0.0, 1.0, 0.0); 
 yellowCubeTransf->addChild(entity->label(), (SceneGraph::cNode*)entity);

 cubeTransf->addChild(yellowCubeTransf->label(), (SceneGraph::cNode*)yellowCubeTransf);


/* RED CUBE */

 //entity
 entity = new SceneGraph::cEntity("redCube", (SceneGraph::cShape*)cube, material);
 
 //texture
 dado = NULL;
 dado = texList["dado3"];
 entity->SetTexture(dado);

 //transform
 SceneGraph::cTransform* redCubeTransf = new SceneGraph::cTransform("redCubeTransf");
 redCubeTransf->Translate(0.0, dim, -(0.6*(dim/2))); 
 redCubeTransf->addChild(entity->label(), (SceneGraph::cNode*)entity);

 cubeTransf->addChild(redCubeTransf->label(), (SceneGraph::cNode*)redCubeTransf);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createSphere(void)
{
 //shape
 SceneGraph::cSphere* sphere = new SceneGraph::cSphere(radius, 32, 32);
 shapeList["sphere"] = (SceneGraph::cShape*)sphere;

 //entity
 SceneGraph::cEntity* entity = new SceneGraph::cEntity("sphere", (SceneGraph::cShape*)sphere,
                                                       (SceneGraph::cAppearance*)appList["sphereMaterial"]);

 //transform
 SceneGraph::cTransform* sphereTransf = new SceneGraph::cTransform("sphereTransf");
 sphereTransf->Translate(-0.7*(dxTable/2), (dyTable/2)+(0.9*radius), 0.7*(dzTable/2)); 
 sphereTransf->addChild(entity->label(), (SceneGraph::cNode*)entity);

 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");

 SceneGraph::cTransform* topTransf = NULL;
 if(tableTransf)
  topTransf = (SceneGraph::cTransform*)tableTransf->getChild("topTransf");

 if(topTransf)
  topTransf->addChild(sphereTransf->label(), (SceneGraph::cNode*)sphereTransf);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createTable(void)
{
 hTable = 40.0;

 dxTable = 50.0;
 dyTable = 5.0;
 dzTable = 100.0;

 dxFoot = 10.0;
 dyFoot = hTable;
 dzFoot = 10.0;

 //tableMaterial
 SceneGraph::cAppearance* material = NULL;
 material = (SceneGraph::cAppearance*)appList["tableMaterial"];

 //tableTexture
 SceneGraph::cTexture* wood = NULL;
 wood = texList["wood"];

 SceneGraph::cTexture* wood1 = NULL;
 wood1 = texList["wood1"];

 //table transform
 SceneGraph::cTransform* tableTransf = new SceneGraph::cTransform("tableTransf");
 SG->addChild(tableTransf->label(), (SceneGraph::cNode*)tableTransf);

 //top
 SceneGraph::cRect* top = new SceneGraph::cRect(dxTable, dyTable, dzTable);
 
 SceneGraph::cAppearance* app = NULL;
 app = appList["tableMaterial"];
 SceneGraph::cEntity* topEnt = new SceneGraph::cEntity("topEnt", (SceneGraph::cShape*)top, material);
 topEnt->SetTexture(wood1);

 SceneGraph::cTransform* topTransf = new SceneGraph::cTransform("topTransf");
 topTransf->addChild(topEnt->label(), (SceneGraph::cNode*)topEnt);

 topTransf->Translate(0.0, hTable, 0.0);

 tableTransf->addChild(topTransf->label(), (SceneGraph::cNode*)topTransf);

 //first table foot
 SceneGraph::cRect* firstFoot = new SceneGraph::cRect(dxFoot, dyFoot, dzFoot);
 
 SceneGraph::cEntity* firstFootEnt = new SceneGraph::cEntity("firstFootEnt", (SceneGraph::cShape*)firstFoot, material);
 firstFootEnt->SetTexture(wood);

 SceneGraph::cTransform* firstFootTransf = new SceneGraph::cTransform("firstFootTransf");
 firstFootTransf->addChild(firstFootEnt->label(), (SceneGraph::cNode*)firstFootEnt);

 firstFootTransf->Translate((dxTable/2.0)-(dxFoot), dyFoot/2.0, (dzTable/2.0)-(dzFoot));

 tableTransf->addChild(firstFootTransf->label(), (SceneGraph::cNode*)firstFootTransf);

 //second table foot
 SceneGraph::cRect* secondFoot = new SceneGraph::cRect(dxFoot, dyFoot, dzFoot);
 
 SceneGraph::cEntity* secondFootEnt = new SceneGraph::cEntity("secondFootEnt", (SceneGraph::cShape*)secondFoot, material);
 secondFootEnt->SetTexture(wood);

 SceneGraph::cTransform* secondFootTransf = new SceneGraph::cTransform("secondFootTransf");
 secondFootTransf->addChild(secondFootEnt->label(), (SceneGraph::cNode*)secondFootEnt);

 secondFootTransf->Translate(-((dxTable/2.0)-(dxFoot)), dyFoot/2.0, (dzTable/2.0)-(dzFoot));

 tableTransf->addChild(secondFootTransf->label(), (SceneGraph::cNode*)secondFootTransf);

 //third table foot
 SceneGraph::cRect* thirdFoot = new SceneGraph::cRect(dxFoot, dyFoot, dzFoot);
 
 SceneGraph::cEntity* thirdFootEnt = new SceneGraph::cEntity("thirdFootEnt", (SceneGraph::cShape*)thirdFoot, material);
 thirdFootEnt->SetTexture(wood);

 SceneGraph::cTransform* thirdFootTransf = new SceneGraph::cTransform("thirdFootTransf");
 thirdFootTransf->addChild(thirdFootEnt->label(), (SceneGraph::cNode*)thirdFootEnt);

 thirdFootTransf->Translate(-((dxTable/2.0)-(dxFoot)), dyFoot/2.0, -((dzTable/2.0)-(dzFoot)));

 tableTransf->addChild(thirdFootTransf->label(), (SceneGraph::cNode*)thirdFootTransf);

 //fourth table foot
 SceneGraph::cRect* fourthFoot = new SceneGraph::cRect(dxFoot, dyFoot, dzFoot);
 
 SceneGraph::cEntity* fourthFootEnt = new SceneGraph::cEntity("fourthFootEnt", (SceneGraph::cShape*)fourthFoot, material);
 fourthFootEnt->SetTexture(wood);

 SceneGraph::cTransform* fourthFootTransf = new SceneGraph::cTransform("fourthFootTransf");
 fourthFootTransf->addChild(fourthFootEnt->label(), (SceneGraph::cNode*)fourthFootEnt);

 fourthFootTransf->Translate(((dxTable/2.0)-(dxFoot)), dyFoot/2.0, -((dzTable/2.0)-(dzFoot)));

 tableTransf->addChild(fourthFootTransf->label(), (SceneGraph::cNode*)fourthFootTransf);

}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createDesktop(void)
{
 //desktop
 SceneGraph::cRect* desktop = new SceneGraph::cRect(20.0, 1.0, 30.0);

 SceneGraph::cAppearance* app = appList["desktopMaterial"];

 SceneGraph::cEntity* desktopEnt = new SceneGraph::cEntity("desktop", (SceneGraph::cShape*)desktop, app);

 SceneGraph::cTransform* desktopTransf = new SceneGraph::cTransform("desktopTransf");
 desktopTransf->addChild(desktopEnt->label(), (SceneGraph::cNode*)desktopEnt);
 desktopTransf->Translate(-(0.2*dxTable), dyTable/2.0 + 0.5, 0.0);

 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");

 SceneGraph::cTransform* topTransf = NULL;
 if(tableTransf)
  topTransf = (SceneGraph::cTransform*)tableTransf->getChild("topTransf");

 if(topTransf)
  topTransf->addChild(desktopTransf->label(), (SceneGraph::cNode*)desktopTransf);


 //paper
 SceneGraph::cRect* paper = new SceneGraph::cRect(18.0, 0.0, 28.0);

 app = appList["paperMaterial"];

 SceneGraph::cEntity* paperEnt = new SceneGraph::cEntity("paper", (SceneGraph::cShape*)paper, app);

 SceneGraph::cTransform* paperTransf = new SceneGraph::cTransform("paperTransf");
 paperTransf->Translate(0.0, 0.6, 0.0);
 paperTransf->addChild(paperEnt->label(), (SceneGraph::cNode*)paperEnt);

 desktopTransf->addChild(paperTransf->label(), (SceneGraph::cNode*)paperTransf);

 //triangles
 cCoord p1(0.0, 0.0, 0.0);
 cCoord p2(-6.0, 0.0, 0.0);
 cCoord p3(0.0, 0.0, -6.0);
 SceneGraph::cTriangle* t1 = new SceneGraph::cTriangle(p1, p2, p3);

 app = appList["desktopMaterial"];

 //T1
 SceneGraph::cEntity* T1Ent = new SceneGraph::cEntity("T1", (SceneGraph::cShape*)t1, app);

 SceneGraph::cTransform* t1Transf = new SceneGraph::cTransform("t1Transf");
 t1Transf->Translate(10.0, 0.7, 15.0);
 t1Transf->addChild(T1Ent->label(), (SceneGraph::cNode*)T1Ent);

 desktopTransf->addChild(t1Transf->label(), (SceneGraph::cNode*)t1Transf);

 //T2
 SceneGraph::cEntity* T2Ent = new SceneGraph::cEntity("T2", (SceneGraph::cShape*)t1, app);

 SceneGraph::cTransform* t2Transf = new SceneGraph::cTransform("t2Transf");
 t2Transf->Translate(10.0, 0.7, -15.0);
 t2Transf->Rotate(90.0, 0.0, 1.0, 0.0);
 t2Transf->addChild(T2Ent->label(), (SceneGraph::cNode*)T2Ent);

 desktopTransf->addChild(t2Transf->label(), (SceneGraph::cNode*)t2Transf);

 /*//T3
 SceneGraph::cEntity* T3Ent = new SceneGraph::cEntity("T3", (SceneGraph::cShape*)t1, app);

 SceneGraph::cTransform* t3Transf = new SceneGraph::cTransform("t3Transf");
 t3Transf->Translate(-10.0, 0.7, -15.0);
 t3Transf->Rotate(180.0, 0.0, 1.0, 0.0);
 t3Transf->addChild(T3Ent->label(), (SceneGraph::cNode*)T3Ent);

 desktopTransf->addChild(t3Transf->label(), (SceneGraph::cNode*)t3Transf);

 //T4
 SceneGraph::cEntity* T4Ent = new SceneGraph::cEntity("T4", (SceneGraph::cShape*)t1, app);

 SceneGraph::cTransform* t4Transf = new SceneGraph::cTransform("t4Transf");
 t4Transf->Translate(-10.0, 0.7, 15.0);
 t4Transf->Rotate(-90.0, 0.0, 1.0, 0.0);
 t4Transf->addChild(T4Ent->label(), (SceneGraph::cNode*)T4Ent);

 desktopTransf->addChild(t4Transf->label(), (SceneGraph::cNode*)t4Transf);
 */
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createBunny(void)
{
 SceneGraph::cMesh* bunny = NULL;

 string filename = exec_path + "database\\bunny.msh";
 
 std::ifstream f;
 f.clear();
 f.open(filename.c_str(), std::ios::in);

 if(f)
 {
  PopupGauge();
  Ihandle* gauge = IupGetHandle("gauge");
  bunny = new SceneGraph::cMesh(f, gauge, true);
  f.close();
  KillGauge();
 }

 if(!bunny)
  return;
 
 //shape
 shapeList["bunny"] = (SceneGraph::cShape*)bunny;

 //entity
 SceneGraph::cEntity* entity = new SceneGraph::cEntity("bunny", (SceneGraph::cShape*)bunny,
                                                       (SceneGraph::cAppearance*)appList["bunnyMaterial"]);

 //transform
 SceneGraph::cTransform* bunnyTransf = new SceneGraph::cTransform("bunnyTransf");
 
 float ydim = bunny->get_yDim();
 float scale = 2*(radius/ydim);

 bunnyTransf->Rotate(180.0, 0.0, 1.0, 0.0);
 bunnyTransf->Translate(-0.6*(dxTable/2), 0.0, -0.6*(dzTable/2)); 
 bunnyTransf->Scale(scale, scale, scale);
 bunnyTransf->addChild(entity->label(), (SceneGraph::cNode*)entity);

 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");

 SceneGraph::cTransform* topTransf = NULL;
 if(tableTransf)
  topTransf = (SceneGraph::cTransform*)tableTransf->getChild("topTransf");

 if(topTransf)
  topTransf->addChild(bunnyTransf->label(), (SceneGraph::cNode*)bunnyTransf);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void createFloor(void)
{
 //shape
 SceneGraph::cRect* floor = new SceneGraph::cRect(5.0*dxTable, 0.0, 2.5*dzTable);
 
 //material
 SceneGraph::cAppearance* app = NULL;
 app = appList["floorMaterial"];

 //texture
 SceneGraph::cTexture* tex = NULL;
 tex = texList["checker"];

 SceneGraph::cEntity* floorEnt = new SceneGraph::cEntity("floorEnt", (SceneGraph::cShape*)floor, app);
 floorEnt->SetTexture(tex);

 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");
 if(tableTransf)
  tableTransf->addChild(floorEnt->label(), (SceneGraph::cNode*)floorEnt);

 //wall texture
 tex = NULL;
 tex = texList["wall"];

 //wall1
 SceneGraph::cEntity* wallEnt = new SceneGraph::cEntity("wall1Ent", (SceneGraph::cShape*)floor, app);
 wallEnt->SetTexture(tex);

 SceneGraph::cTransform* wallTransf = new SceneGraph::cTransform("wall1Transf");
 wallTransf->addChild(wallEnt->label(), (SceneGraph::cNode*)wallEnt);
 wallTransf->Translate(-(5.0*dxTable)/2.0, (2.5*dzTable)/2, 0.0);
 wallTransf->Rotate(90.0, 0.0, 0.0, 1.0);

 if(tableTransf)
  tableTransf->addChild(wallTransf->label(), (SceneGraph::cNode*)wallTransf);

 //window

 //shape
 SceneGraph::cRect* window = new SceneGraph::cRect(0.4*5.0*dxTable, 0.0, 0.4*2.5*dzTable);
 SceneGraph::cEntity* windowEnt = new SceneGraph::cEntity("windowEnt", (SceneGraph::cShape*)window, app);
 
 tex = texList["window"];
 tex->mode(SceneGraph::cTexture::REPLACE);
 windowEnt->SetTexture(tex);
 

 SceneGraph::cTransform* windowTransf = new SceneGraph::cTransform("windowTransf");
 windowTransf->Translate(0.0, -0.5, 10.0);
 windowTransf->Rotate(90.0, 0.0, 1.0, 0.0);
 windowTransf->addChild(windowEnt->label(), (SceneGraph::cNode*)windowEnt);

 wallTransf->addChild(windowTransf->label(), (SceneGraph::cNode*)windowTransf);

 tex = texList["wall"];

 //wall2
 wallEnt = new SceneGraph::cEntity("wall2Ent", (SceneGraph::cShape*)floor, app);
 wallEnt->SetTexture(tex);

 wallTransf = new SceneGraph::cTransform("wall2Transf");
 wallTransf->addChild(wallEnt->label(), (SceneGraph::cNode*)wallEnt);
 wallTransf->Translate(0.0, (2.5*dzTable)/2, (2.5*dzTable)/2);
 wallTransf->Rotate(90.0, 1.0, 0.0, 0.0);

 if(tableTransf)
  tableTransf->addChild(wallTransf->label(), (SceneGraph::cNode*)wallTransf);

 //picture

 //shape
 SceneGraph::cRect* picture = new SceneGraph::cRect(0.4*5.0*dxTable, 0.0, 0.4*2.5*dzTable);
 SceneGraph::cEntity* pictureEnt = new SceneGraph::cEntity("pictureEnt", (SceneGraph::cShape*)picture, app);
 
 tex = texList["picture"];
 tex->mode(SceneGraph::cTexture::REPLACE);
 pictureEnt->SetTexture(tex);
 

 SceneGraph::cTransform* pictureTransf = new SceneGraph::cTransform("pictureTransf");
 pictureTransf->Translate(0.0, -0.5, 10.0);
 pictureTransf->addChild(pictureEnt->label(), (SceneGraph::cNode*)pictureEnt);

 wallTransf->addChild(pictureTransf->label(), (SceneGraph::cNode*)pictureTransf);

 /*//
 SceneGraph::cRect* picture = new SceneGraph::cRect(0.4*dxTable, 3.0, 0.3*dzTable);
 pictureEnt = new SceneGraph::cEntity("pictureEnt", (SceneGraph::cShape*)picture, app);

 pictureTransf = new SceneGraph::cTransform("pictureTransf");
 pictureTransf->addChild(pictureEnt->label(), (SceneGraph::cNode*)pictureEnt);

 wallTransf->addChild(pictureTransf->label(), (SceneGraph::cNode*)pictureTransf);
*/
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void initSG(void)
{
 //scene
 float clearColor[] = { 0.0, 0.0, 0.0, 1.0 };
 SG = new SceneGraph::cScene("scene", clearColor);
 if(!SG)
  return;

 //camera
 createCamera();

 //light
 createLight();

 //material
 createMaterials();

 //texture
 createTexture();

 //table
 createTable();

  //sphere
 createSphere();

 //cube
 createCubes();

 //bunny
 createBunny();

 //floor
 createFloor();

 //desktop
 createDesktop();

 //table pos
 SceneGraph::cTransform* tableTransf = NULL;
 tableTransf = (SceneGraph::cTransform*)SG->getChild("tableTransf");
 if(tableTransf)
 {
  tableTransf->Rotate(90.0, 0.0, 0.0, 1.0);
  tableTransf->Rotate(90.0, 1.0, 0.0, 0.0);
  tableTransf->Translate(0.0, -80.0, 0.0);
 }

 //cam pos
 SceneGraph::cTransform* camTransf = NULL;
 camTransf = (SceneGraph::cTransform*)SG->getChild("camTransf");
 if(camTransf)
 {
  camTransf->Rotate(30.0, 1.0, 0.0, 0.0);
  camTransf->Rotate(45.0, 0.0, 0.0, 1.0);
 }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void drawText(int x, int y, char * msg)
{
 int i, len;
 glRasterPos2i(x, y);
 len = strlen(msg);
 for(i = 0; i < len; i++)
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, msg[i]);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void drawTitleText(int x, int y, char * msg)
{
 int i, len;
 glRasterPos2i(x, y);
 len = strlen(msg);
 for(i = 0; i < len; i++)
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, msg[i]);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void drawMoveLegend(void)
{
 glBegin(GL_TRIANGLES);
  glVertex2f(3.0, 5.5);
  glVertex2f(5.0, 6.0);
  glVertex2f(5.0, 5.0);
 glEnd();

 glBegin(GL_LINES);
  glVertex2f(5.0, 5.5);
  glVertex2f(8.0, 5.5);
 glEnd();

 glBegin(GL_TRIANGLES);
  glVertex2f(13.0, 9.5);
  glVertex2f(14.0, 8.5);
  glVertex2f(12.0, 8.5);
 glEnd();

 glBegin(GL_LINES);
  glVertex2f(13.0, 8.5);
  glVertex2f(13.0, 7.0);
 glEnd();

 drawText(10.0, 5.8, "Move");

 glBegin(GL_LINES);
  glVertex2f(17.0, 5.5);
  glVertex2f(20.0, 5.5);
 glEnd();

 glBegin(GL_TRIANGLES);
  glVertex2f(22.0, 5.5);
  glVertex2f(20.0, 6.0);
  glVertex2f(20.0, 5.0);
 glEnd();

 glBegin(GL_LINES);
  glVertex2f(13.0, 2.5);
  glVertex2f(13.0, 4.0);
 glEnd();

 glBegin(GL_TRIANGLES);
  glVertex2f(13.0, 1.5);
  glVertex2f(14.0, 2.5);
  glVertex2f(12.0, 2.5);
 glEnd();
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void drawLegend(void)
{
 drawText(68.0, 10.5, "x: ZY Plane");
 drawText(68.0, 8.5, "X: YZ Plane");

 drawText(68.0, 6.5, "y: XZ Plane");
 drawText(68.0, 4.5, "Y: ZX Plane");

 drawText(68.0, 2.5, "z: YX Plane");
 drawText(68.0, 0.5, "Z: XY Plane");

 drawText(82.0, 10.0, "Home: Fit");
 drawText(82.0, 6.0, "PG_Up: Zoom In");
 drawText(82.0, 2.0, "PG_Dw: Zoom Out");
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int redraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 static first = 1;
 if(first)
 {
  initSG();
  first = 0;
 }

 if(SG)
  SG->Render();

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int reshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 left_winWidth = w;
 left_winHeight = h;

 IupRefresh(self);

 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int keyboard(Ihandle* self, int c)
{
 cCoord axial_x(1.0, 0.0, 0.0);
 cCoord axial_y(0.0, 1.0, 0.0);
 cCoord axial_z(0.0, 0.0, 1.0);

 SceneGraph::cTransform* T = NULL;
 T = (SceneGraph::cTransform*)SG->getChild("camTransf");
 if(!T)
  return IUP_IGNORE;

 switch(c)
	{
	 case K_DOWN:
   T->Rotate(5.0, axial_x.x, axial_x.y, axial_x.z);
   break;
  case K_UP:
   T->Rotate(-5.0, axial_x.x, axial_x.y, axial_x.z);
   break;
  case K_RIGHT:
   T->Rotate(5.0, axial_z.x, axial_z.y, axial_z.z);
   break;
  case K_LEFT:
   T->Rotate(-5.0, axial_z.x, axial_z.y, axial_z.z);
   break;
  /*case 'x':
   x_rot = x_rot_init;
   y_rot = y_rot_init + 90.0;
   z_rot = z_rot_init;
   break;
  case 'X':
   x_rot = x_rot_init;
   y_rot = y_rot_init - 90.0;
   z_rot = z_rot_init;
   break;
  case 'y':
   x_rot = x_rot_init - 90.0;
   y_rot = y_rot_init;
   z_rot = z_rot_init;
   break;
  case 'Y':
   x_rot = x_rot_init + 90.0;
   y_rot = y_rot_init;
   z_rot = z_rot_init;
   break;
  case 'z':
   x_rot = x_rot_init;
   y_rot = y_rot_init + 180.0;
   z_rot = z_rot_init;
   break;
  case 'Z':
   x_rot = x_rot_init;
   y_rot = y_rot_init;
   z_rot = z_rot_init;
   break;*/
  case K_PGUP:
   T->Translate(0.0, -10.0, 0.0);
   break;
  case K_PGDN:
   T->Translate(0.0, 10.0, 0.0);
   break;
	}
 IupRefresh(canvas);
	return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void setMainDialogCB(Ihandle* cv)
{
 canvas = cv;

 IupSetFunction("Exit", (Icallback)cbExit);
 //IupSetFunction("OpenTriangles", (Icallback)cbOpenTriangles);
 //IupSetFunction("OpenTetrahedrons", (Icallback)cbOpenTetrahedrons);

 IupSetCallback(canvas, "ACTION", (Icallback)redraw);
 IupSetCallback(canvas, "RESIZE_CB", (Icallback)reshape);
 IupSetCallback(canvas, "KEYPRESS_CB", (Icallback)keyboard);
}
