// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#include "material.h"

namespace SceneGraph
{

//------------------------------------------------------------
//------------------------------------------------------------
cMaterial::cMaterial(float* amb, float* diff, float* spec, float shin)
{
 _shininess = shin;
 for(int i = 0; i < 4; i++)
 {
  _ambient[i] = amb[i];
  _diffuse[i] = diff[i];
  _specular[i] = spec[i];
 }

 _opaque = true;
 if(amb[3] < 1.0 || diff[3] < 1.0 || spec[3] < 1.0)
  _opaque = false;
}

//------------------------------------------------------------
//------------------------------------------------------------
void cMaterial::ambient(float* amb)
{
 for(int i = 0; i < 4; i++)
  _ambient[i] = amb[i];

 if(amb[3] < 1.0)
  _opaque = false;
}

//------------------------------------------------------------
//------------------------------------------------------------
void cMaterial::diffuse(float* diff)
{
 for(int i = 0; i < 4; i++)
  _diffuse[i] = diff[i];

 if(diff[3] < 1.0)
  _opaque = false;
}

//------------------------------------------------------------
//------------------------------------------------------------
void cMaterial::specular(float* spec)
{
 for(int i = 0; i < 4; i++)
  _specular[i] = spec[i];

 if(spec[3] < 1.0)
  _opaque = false;
}

//------------------------------------------------------------
//------------------------------------------------------------
void cMaterial::Load(void)
{
 glPushAttrib(GL_LIGHTING_BIT); 
 glShadeModel(GL_SMOOTH);

 glMaterialfv(GL_FRONT, GL_AMBIENT, _ambient);
 glMaterialfv(GL_FRONT, GL_DIFFUSE, _diffuse);
 glMaterialfv(GL_FRONT, GL_SPECULAR, _specular);
 glMaterialf(GL_FRONT, GL_SHININESS, _shininess);
}

//------------------------------------------------------------
//------------------------------------------------------------
void cMaterial::Unload(void)
{
 glPopAttrib();
}

}
