// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>

#include <float.h>

#include "mesh.h"
#include "util.h"

namespace SceneGraph
{

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cMesh::cMesh(std::ifstream &f, Ihandle* gauge, bool tr) : vecVertex(0), vecTria(0), vecQuad(0),
vertexTria(0), vertexQuad(0), normalTria(0), normalQuad(0), indexTria(0), indexQuad(0), x_translate(0.0),
y_translate(0.0), z_translate(0.0), scale(1.0)
{
 if(tr)
 {
  ReadTriaFile(f, gauge);
  SetSmoothGlVector3();
 }
 else
 {
  ReadQuadFile(f);
  SetGlVector4();
 }
 tria = tr;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cMesh::cMesh(const cMesh& obj, bool tr) : vecVertex(0), vecTria(0), vecQuad(0), vertexTria(0),
vertexQuad(0), normalTria(0), normalQuad(0), indexTria(0), indexQuad(0)
{
 if(tr)
 {
  CopyTriaObj(obj);
  SetGlVector3();
 }
 else
 {
  CopyQuadObj(obj);
  SetGlVector4();
 }
 tria = tr;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::CopyTriaObj(const cMesh& obj)
{
 string s, msg = "";
 cCoord vec1, vec2;
 int v1, v2, v3;
 int i;
 
 sVertex coord;
 sFace3v tria;
 
 const float limit = FLT_MAX;

 float xmin =  limit;
 float xmax = -limit;
 float ymin =  limit;
 float ymax = -limit;
 float zmin =  limit;
 float zmax = -limit;

 numvertex = obj.numvertex;
 numtria = obj.numtria;
 for(i = 0; i < numvertex; i++)
 {
  coord.v.x = obj.vecVertex[i].v.x;
  coord.v.y = obj.vecVertex[i].v.y;
  coord.v.z = obj.vecVertex[i].v.z;

  coord.normal = 0.0;
  vecVertex.push_back(coord);
 }
 for(i = 0; i < numtria; i++)
 {
  tria.v1 = obj.vecTria[i].v1;
  tria.v2 = obj.vecTria[i].v2;
  tria.v3 = obj.vecTria[i].v3;

  v1 = tria.v1;
  v2 = tria.v2;
  v3 = tria.v3;
  
  vec1 = vecVertex[v1].v - vecVertex[v2].v;
  vec2 = vecVertex[v3].v - vecVertex[v2].v;
  tria.normal = CrossProd(vec2, vec1);
  vecTria.push_back(tria);

  xmin = Min(xmin, vecVertex[v1].v.x);
  xmax = Max(xmax, vecVertex[v1].v.x);
  ymin = Min(ymin, vecVertex[v1].v.y);
  ymax = Max(ymax, vecVertex[v1].v.y);
  zmin = Min(zmin, vecVertex[v1].v.z);
  zmax = Max(zmax, vecVertex[v1].v.z);

  xmin = Min(xmin, vecVertex[v2].v.x);
  xmax = Max(xmax, vecVertex[v2].v.x);
  ymin = Min(ymin, vecVertex[v2].v.y);
  ymax = Max(ymax, vecVertex[v2].v.y);
  zmin = Min(zmin, vecVertex[v2].v.z);
  zmax = Max(zmax, vecVertex[v2].v.z);

  xmin = Min(xmin, vecVertex[v3].v.x);
  xmax = Max(xmax, vecVertex[v3].v.x);
  ymin = Min(ymin, vecVertex[v3].v.y);
  ymax = Max(ymax, vecVertex[v3].v.y);
  zmin = Min(zmin, vecVertex[v3].v.z);
  zmax = Max(zmax, vecVertex[v3].v.z);
 }
 
 scale = obj.scale;

 x_translate = obj.x_translate;
 y_translate = obj.y_translate;
 z_translate = obj.z_translate;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::CopyQuadObj(const cMesh& obj)
{
 string s, msg = "";
 cCoord vec1, vec2;
 int v1, v2, v3, v4;
 int i;
 
 sVertex coord;
 sFace4v quad;
 
 const float limit = FLT_MAX;

 float xmin =  limit;
 float xmax = -limit;
 float ymin =  limit;
 float ymax = -limit;
 float zmin =  limit;
 float zmax = -limit;

 numvertex = obj.numvertex;
 numquad = obj.numquad;
 for(i = 0; i < numvertex; i++)
 {
  coord.v.x = obj.vecVertex[i].v.x;
  coord.v.y = obj.vecVertex[i].v.y;
  coord.v.z = obj.vecVertex[i].v.z;

  coord.normal = 0.0;
  vecVertex.push_back(coord);
 }
 for(i = 0; i < numquad; i++)
 {
  quad.v1 = obj.vecQuad[i].v1;
  quad.v2 = obj.vecQuad[i].v2;
  quad.v3 = obj.vecQuad[i].v3;
  quad.v4 = obj.vecQuad[i].v4;

  v1 = quad.v1;
  v2 = quad.v2;
  v3 = quad.v3;
  v4 = quad.v4;
  
  vec1 = vecVertex[v1].v - vecVertex[v2].v;
  vec2 = vecVertex[v3].v - vecVertex[v2].v;
  quad.normal = CrossProd(vec2, vec1);
  vecQuad.push_back(quad);

  xmin = Min(xmin, vecVertex[v1].v.x);
  xmax = Max(xmax, vecVertex[v1].v.x);
  ymin = Min(ymin, vecVertex[v1].v.y);
  ymax = Max(ymax, vecVertex[v1].v.y);
  zmin = Min(zmin, vecVertex[v1].v.z);
  zmax = Max(zmax, vecVertex[v1].v.z);

  xmin = Min(xmin, vecVertex[v2].v.x);
  xmax = Max(xmax, vecVertex[v2].v.x);
  ymin = Min(ymin, vecVertex[v2].v.y);
  ymax = Max(ymax, vecVertex[v2].v.y);
  zmin = Min(zmin, vecVertex[v2].v.z);
  zmax = Max(zmax, vecVertex[v2].v.z);

  xmin = Min(xmin, vecVertex[v3].v.x);
  xmax = Max(xmax, vecVertex[v3].v.x);
  ymin = Min(ymin, vecVertex[v3].v.y);
  ymax = Max(ymax, vecVertex[v3].v.y);
  zmin = Min(zmin, vecVertex[v3].v.z);
  zmax = Max(zmax, vecVertex[v3].v.z);

  xmin = Min(xmin, vecVertex[v4].v.x);
  xmax = Max(xmax, vecVertex[v4].v.x);
  ymin = Min(ymin, vecVertex[v4].v.y);
  ymax = Max(ymax, vecVertex[v4].v.y);
  zmin = Min(zmin, vecVertex[v4].v.z);
  zmax = Max(zmax, vecVertex[v4].v.z);
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::ReadTriaFile(std::ifstream &f, Ihandle* gauge)
{
 string s, msg = "";
 int v1, v2, v3;
 int i, dummy;
 
 cCoord vec1, vec2, normal;
 float modulo1, modulo2;

 sVertex coord;
 sFace3v tria;
 
 char aux[10];

 float file_length = 0.0f;
 if(gauge)
 {
  f.seekg(0, std::ios::end);
  file_length = (float)f.tellg();
  f.seekg(0, std::ios::beg);
 }
 
 const float limit = FLT_MAX;

 float xmin =  limit;
 float xmax = -limit;
 float ymin =  limit;
 float ymax = -limit;
 float zmin =  limit;
 float zmax = -limit;

 f >> numvertex;
 f >> numtria;
 for(i = 0; i < numvertex; i++)
 {
  f >> dummy;
  f >> coord.v.x;
  f >> coord.v.y;
  f >> coord.v.z;

  coord.normal = 0.0;
  vecVertex.push_back(coord);

  if(gauge)
  {
   sprintf(aux, "%.3f", (float)f.tellg()/file_length);
   IupSetAttribute(gauge, IUP_VALUE, aux);
  }
 }
 for(i = 0; i < numtria; i++)
 {
  f >> dummy;
  f >> tria.v1;
  f >> tria.v2;
  f >> tria.v3;

  v1 = tria.v1;
  v2 = tria.v2;
  v3 = tria.v3;
  vecTria.push_back(tria);

  //normal em v1
  vec1 = vecVertex[v2].v - vecVertex[v1].v;
  vec2 = vecVertex[v3].v - vecVertex[v1].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v1].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v2
  vec1 = vecVertex[v3].v - vecVertex[v2].v;
  vec2 = vecVertex[v1].v - vecVertex[v2].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v2].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v3
  vec1 = vecVertex[v1].v - vecVertex[v3].v;
  vec2 = vecVertex[v2].v - vecVertex[v3].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v3].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  xmin = Min(xmin, vecVertex[v1].v.x);
  xmax = Max(xmax, vecVertex[v1].v.x);
  ymin = Min(ymin, vecVertex[v1].v.y);
  ymax = Max(ymax, vecVertex[v1].v.y);
  zmin = Min(zmin, vecVertex[v1].v.z);
  zmax = Max(zmax, vecVertex[v1].v.z);

  xmin = Min(xmin, vecVertex[v2].v.x);
  xmax = Max(xmax, vecVertex[v2].v.x);
  ymin = Min(ymin, vecVertex[v2].v.y);
  ymax = Max(ymax, vecVertex[v2].v.y);
  zmin = Min(zmin, vecVertex[v2].v.z);
  zmax = Max(zmax, vecVertex[v2].v.z);

  xmin = Min(xmin, vecVertex[v3].v.x);
  xmax = Max(xmax, vecVertex[v3].v.x);
  ymin = Min(ymin, vecVertex[v3].v.y);
  ymax = Max(ymax, vecVertex[v3].v.y);
  zmin = Min(zmin, vecVertex[v3].v.z);
  zmax = Max(zmax, vecVertex[v3].v.z);

  if(gauge)
  {
   sprintf(aux, "%.3f", (float)f.tellg()/file_length);
   IupSetAttribute(gauge, IUP_VALUE, aux);
  }

 }

 x_min = xmin;
 x_max = xmax;
 y_min = ymin;
 y_max = ymax;
 z_min = zmin;
 z_max = zmax;
 
 x_translate = -(x_min + ((xmax - xmin)/2));
 y_translate = -(y_min + ((ymax - ymin)/2));
 z_translate = -(z_min + ((zmax - zmin)/2));

 scale = (float)( (tan(((45*MPI)/360)*(60.0-z_max)) / y_max ));
}

 // ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::ReadQuadFile(std::ifstream &f)
{
 string s, msg = "";
 int v1, v2, v3, v4;
 int i, dummy;
 
 cCoord vec1, vec2, normal;

 sVertex coord;
 sFace4v quad;
 
 char aux[10];
 
 const float limit = FLT_MAX;

 float xmin =  limit;
 float xmax = -limit;
 float ymin =  limit;
 float ymax = -limit;
 float zmin =  limit;
 float zmax = -limit;

 f >> numvertex;
 f >> numquad;
 for(i = 0; i < numvertex; i++)
 {
  f >> dummy;
  f >> coord.v.x;
  f >> coord.v.y;
  f >> coord.v.z;

  coord.normal = 0.0;
  vecVertex.push_back(coord);

 }
 for(i = 0; i < numquad; i++)
 {
  f >> dummy;
  f >> quad.v1;
  f >> quad.v2;
  f >> quad.v3;
  f >> quad.v4;

  v1 = quad.v1;
  v2 = quad.v2;
  v3 = quad.v3;
  v4 = quad.v4;
    
  //normal em v1
  vec1 = vecVertex[v2].v - vecVertex[v1].v;
  vec2 = vecVertex[v3].v - vecVertex[v1].v;
  quad.normal = CrossProd(vec1, vec2);
  vecQuad.push_back(quad);
  /*modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v1].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

/*  //normal em v1
  vec1 = vecVertex[v3].v - vecVertex[v1].v;
  vec2 = vecVertex[v4].v - vecVertex[v1].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v1].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v1
  vec1 = vecVertex[v4].v - vecVertex[v1].v;
  vec2 = vecVertex[v2].v - vecVertex[v1].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v1].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v2
  vec1 = vecVertex[v2].v - vecVertex[v2].v;
  vec2 = vecVertex[v3].v - vecVertex[v2].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v2].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

/*  //normal em v2
  vec1 = vecVertex[v3].v - vecVertex[v2].v;
  vec2 = vecVertex[v4].v - vecVertex[v2].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v2].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v2
  vec1 = vecVertex[v4].v - vecVertex[v2].v;
  vec2 = vecVertex[v2].v - vecVertex[v2].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v2].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v3
  vec1 = vecVertex[v2].v - vecVertex[v3].v;
  vec2 = vecVertex[v3].v - vecVertex[v3].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v3].normal += (normal*(normal.modulo()/(modulo1*modulo2)));
/*
  //normal em v3
  vec1 = vecVertex[v3].v - vecVertex[v3].v;
  vec2 = vecVertex[v4].v - vecVertex[v3].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v3].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v3
  vec1 = vecVertex[v4].v - vecVertex[v3].v;
  vec2 = vecVertex[v2].v - vecVertex[v3].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v3].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v4
  vec1 = vecVertex[v2].v - vecVertex[v4].v;
  vec2 = vecVertex[v3].v - vecVertex[v4].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v4].normal += (normal*(normal.modulo()/(modulo1*modulo2)));
/*
  //normal em v4
  vec1 = vecVertex[v3].v - vecVertex[v4].v;
  vec2 = vecVertex[v4].v - vecVertex[v4].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v4].normal += (normal*(normal.modulo()/(modulo1*modulo2)));

  //normal em v4
  vec1 = vecVertex[v4].v - vecVertex[v4].v;
  vec2 = vecVertex[v2].v - vecVertex[v4].v;
  modulo1 = vec1.modulo();
  modulo2 = vec2.modulo();
  normal = CrossProd(vec1, vec2);
  vecVertex[v4].normal += (normal*(normal.modulo()/(modulo1*modulo2)));
*/
  xmin = Min(xmin, vecVertex[v1].v.x);
  xmax = Max(xmax, vecVertex[v1].v.x);
  ymin = Min(ymin, vecVertex[v1].v.y);
  ymax = Max(ymax, vecVertex[v1].v.y);
  zmin = Min(zmin, vecVertex[v1].v.z);
  zmax = Max(zmax, vecVertex[v1].v.z);

  xmin = Min(xmin, vecVertex[v2].v.x);
  xmax = Max(xmax, vecVertex[v2].v.x);
  ymin = Min(ymin, vecVertex[v2].v.y);
  ymax = Max(ymax, vecVertex[v2].v.y);
  zmin = Min(zmin, vecVertex[v2].v.z);
  zmax = Max(zmax, vecVertex[v2].v.z);

  xmin = Min(xmin, vecVertex[v3].v.x);
  xmax = Max(xmax, vecVertex[v3].v.x);
  ymin = Min(ymin, vecVertex[v3].v.y);
  ymax = Max(ymax, vecVertex[v3].v.y);
  zmin = Min(zmin, vecVertex[v3].v.z);
  zmax = Max(zmax, vecVertex[v3].v.z);

  xmin = Min(xmin, vecVertex[v4].v.x);
  xmax = Max(xmax, vecVertex[v4].v.x);
  ymin = Min(ymin, vecVertex[v4].v.y);
  ymax = Max(ymax, vecVertex[v4].v.y);
  zmin = Min(zmin, vecVertex[v4].v.z);
  zmax = Max(zmax, vecVertex[v4].v.z);
 }
 
// y_delta = (fabs(ymax) - fabs(ymin))/2;

}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::FreeMesh(void)
{
 if(vertexTria) delete[] vertexTria;
 if(indexTria) delete[] indexTria;
 if(normalTria) delete[] normalTria;

 if(vertexQuad) delete[] vertexQuad;
 if(indexQuad) delete[] indexQuad;
 if(normalQuad) delete[] normalQuad;

 vecVertex.clear();
 vecTria.clear();
 vecQuad.clear();
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::SetGlVector3(void)
{
 if(vertexTria)
 {
  delete[] vertexTria;
  delete[] normalTria;
  delete[] indexTria;
 }

 int i, j, k, v1, v2, v3;
 
 numtria = (int)vecTria.size();
 if(numtria == 0) return;

 vertexTria = new float[3*3*numtria];
 normalTria = new float[3*3*numtria];
 indexTria  = new unsigned int[3*numtria];

 k = 0;
 for(i=0; i<numtria; i++)
 {
  v1 = vecTria[i].v1;
  v2 = vecTria[i].v2;
  v3 = vecTria[i].v3;

  j = 3 * k;
  vertexTria[j]   = vecVertex[v1].v.x;
  vertexTria[j+1] = vecVertex[v1].v.y;
  vertexTria[j+2] = vecVertex[v1].v.z;
  normalTria[j]   = vecTria[i].normal.x;
  normalTria[j+1] = vecTria[i].normal.y;
  normalTria[j+2] = vecTria[i].normal.z;
  indexTria[k] = k;

  k++;
  j = 3 * k;
  vertexTria[j]   = vecVertex[v2].v.x;
  vertexTria[j+1] = vecVertex[v2].v.y;
  vertexTria[j+2] = vecVertex[v2].v.z;
  normalTria[j]   = vecTria[i].normal.x;
  normalTria[j+1] = vecTria[i].normal.y;
  normalTria[j+2] = vecTria[i].normal.z;
  indexTria[k] = k;

  k++;
  j = 3 * k;
  vertexTria[j]   = vecVertex[v3].v.x;
  vertexTria[j+1] = vecVertex[v3].v.y;
  vertexTria[j+2] = vecVertex[v3].v.z;
  normalTria[j]   = vecTria[i].normal.x;
  normalTria[j+1] = vecTria[i].normal.y;
  normalTria[j+2] = vecTria[i].normal.z;
  indexTria[k] = k;
  k++;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::SetSmoothGlVector3(void)
{
 if(vertexTria)
 {
  delete[] vertexTria;
  delete[] normalTria;
  delete[] indexTria;
 }

 int i, j;
 
 numtria = (int)vecTria.size();
 if(numtria == 0) return;

 vertexTria = new float[3*numvertex];
 normalTria = new float[3*numvertex];

 for(i = 0; i < numvertex; i++)
 {
  j = 3 * i;
  vertexTria[j]   = vecVertex[i].v.x;
  vertexTria[j+1] = vecVertex[i].v.y;
  vertexTria[j+2] = vecVertex[i].v.z;
  normalTria[j]   = vecVertex[i].normal.x;
  normalTria[j+1] = vecVertex[i].normal.y;
  normalTria[j+2] = vecVertex[i].normal.z;
 }

 indexTria  = new unsigned int[3*numtria];

 for(i = 0; i < numtria; i++)
 {
  j = 3 * i;
  indexTria[j] = vecTria[i].v1;
  indexTria[j+1] = vecTria[i].v2;
  indexTria[j+2] = vecTria[i].v3;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh :: SetGlVector4(void)
{
 if(vertexQuad)
 {
  delete[] vertexQuad;
  delete[] normalQuad;
  delete[] indexQuad;
 }

 int i, j, k, v1, v2, v3, v4;
 
 numquad = (int)vecQuad.size();
 if(numquad == 0) return;

 vertexQuad = new float[3*4*numquad];
 normalQuad = new float[3*4*numquad];
 indexQuad  = new unsigned int[4*numquad];

 k = 0;
 for(i=0; i<numquad; i++)
 {
  v1 = vecQuad[i].v1;
  v2 = vecQuad[i].v2;
  v3 = vecQuad[i].v3;
  v4 = vecQuad[i].v4;

  j = 3 * k;
  vertexQuad[j]   = vecVertex[v1].v.x;
  vertexQuad[j+1] = vecVertex[v1].v.y;
  vertexQuad[j+2] = vecVertex[v1].v.z;
  normalQuad[j]   = vecQuad[i].normal.x;
  normalQuad[j+1] = vecQuad[i].normal.y;
  normalQuad[j+2] = vecQuad[i].normal.z;
  indexQuad[k] = k;

  k++;
  j = 3 * k;
  vertexQuad[j]   = vecVertex[v2].v.x;
  vertexQuad[j+1] = vecVertex[v2].v.y;
  vertexQuad[j+2] = vecVertex[v2].v.z;
  normalQuad[j]   = vecQuad[i].normal.x;
  normalQuad[j+1] = vecQuad[i].normal.y;
  normalQuad[j+2] = vecQuad[i].normal.z;
  indexQuad[k] = k;

  k++;
  j = 3 * k;
  vertexQuad[j]   = vecVertex[v3].v.x;
  vertexQuad[j+1] = vecVertex[v3].v.y;
  vertexQuad[j+2] = vecVertex[v3].v.z;
  normalQuad[j]   = vecQuad[i].normal.x;
  normalQuad[j+1] = vecQuad[i].normal.y;
  normalQuad[j+2] = vecQuad[i].normal.z;
  indexQuad[k] = k;

  k++;
  j = 3 * k;
  vertexQuad[j]   = vecVertex[v4].v.x;
  vertexQuad[j+1] = vecVertex[v4].v.y;
  vertexQuad[j+2] = vecVertex[v4].v.z;
  normalQuad[j]   = vecQuad[i].normal.x;
  normalQuad[j+1] = vecQuad[i].normal.y;
  normalQuad[j+2] = vecQuad[i].normal.z;
  indexQuad[k] = k;

  k++;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cMesh::draw(void)
{
 glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_POLYGON_BIT | GL_LIGHTING_BIT);
 glEnableClientState(GL_VERTEX_ARRAY);
 glEnableClientState(GL_NORMAL_ARRAY);
 glEnable(GL_NORMALIZE);

 if(tria)
 {
  glNormalPointer(GL_FLOAT, 0, normalTria);
  glVertexPointer(3, GL_FLOAT, 0, vertexTria);
  glDrawElements(GL_TRIANGLES, 3*numtria, GL_UNSIGNED_INT, indexTria);
 }
 else
 {
  glNormalPointer(GL_FLOAT, 0, normalQuad);
  glVertexPointer(3, GL_FLOAT, 0, vertexQuad);
  glDrawElements(GL_QUADS, 4*numquad, GL_UNSIGNED_INT, indexQuad);
 }

 glDisable(GL_NORMALIZE);
 glPopAttrib();
}

}
