// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _MESH_H_
#define _MESH_H_

#include <vector>
#include <fstream>
#include <string>
#include <iup.h>

#include "coord.h"
#include "shape.h"

using std::vector;
using std::ifstream;
using std::string;

namespace SceneGraph
{

struct sVertex
{
 cCoord v;
 cCoord normal;
};

struct sFace3v
{
 int v1, v2, v3;
 cCoord normal;
};

struct sFace4v
{
 int v1, v2, v3, v4;
 cCoord normal;
};


class cMesh : public cShape
{
 private:
  int numvertex, numtria, numquad;

  float scale;

  float x_translate;
  float y_translate;
  float z_translate;

  float x_min;
  float x_max;
  float y_min;
  float y_max;
  float z_min;
  float z_max;

  bool tria;
  
  // variaveis de leitura do arquivo
  vector<sVertex> vecVertex;
  vector<sFace3v> vecTria;
  vector<sFace4v> vecQuad;

  
  // variaveis ja processadas para desenho no OpenGL
  float *vertexTria, *vertexQuad;
  float *normalTria, *normalQuad;
  unsigned int *indexTria, *indexQuad;

  void CopyTriaObj(const cMesh&);
  void CopyQuadObj(const cMesh&);

  void ReadTriaFile(std::ifstream &f, Ihandle*);
  void ReadQuadFile(std::ifstream &f);

  void FreeMesh(void);
  
  void SetGlVector3(void);
  void SetGlVector4(void);

  void SetSmoothGlVector3(void);
  
 public:
  cMesh(std::ifstream&, Ihandle*, bool);
  cMesh(const cMesh&, bool);
  ~cMesh(void) {FreeMesh();}

  float get_yDim(void) { return (y_max - y_min); }

  void draw(void);
};


}

#endif
