// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#include "scene.h"

namespace SceneGraph
{

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
 cScene::cScene(string label, float* rgba) : cGroup(label, cNode::SCENE)
{
 for(int i = 0; i < 4; i++)
  _clearColor[i] = rgba[i];
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cScene::cScene(string label, map<string, cNode*> vec, float* rgba) : cGroup(label, cNode::SCENE, vec)
{
 for(int i = 0; i < 4; i++)
  _clearColor[i] = rgba[i];
}

//------------------------------------------------------------
//------------------------------------------------------------
int cScene::SetupCamera(void)
{
 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();
 for(it = b; it != e; it++)
 {
  if(it->second->SetupCamera())
   return 1;
 }
 return 0;
}

//------------------------------------------------------------
//------------------------------------------------------------
int cScene::SetupLights(void)
{
 int n = 0;
 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();
 for(it = b; it != e; it++)
  n += it->second->SetupLights();

 return n;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cScene::Render(void)
{
 glClearColor(_clearColor[0], _clearColor[1], _clearColor[2], _clearColor[3]);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 glEnable(GL_DEPTH_TEST);
 //glEnable(GL_COLOR_MATERIAL);
 
 //search graph to set cam
 this->SetupCamera();

 //search graph to set lights
 if(this->SetupLights())
  glEnable(GL_LIGHTING);

 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();

 //first opaque objs
 for(it = b; it != e; it++)
  it->second->Render(true);

 //then transparency objs
 for(it = b; it != e; it++)
  it->second->Render(false);
}

}