// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _SHAPE_H_
#define _SHAPE_H_

#include "coord.h"

namespace SceneGraph
{

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cShape
{
private:
 bool _draw;

public:
 cShape() {}
 ~cShape() {}

 virtual void draw(void) = 0;

 virtual void drawTexture(void) { return; }
};

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cTriangle : public cShape
{
private:
 cCoord _p1, _p2, _p3;

public:
 cTriangle(cCoord p1, cCoord p2, cCoord p3) : cShape(), _p1(p1), _p2(p2), _p3(p3) {}

 void setPoints(cCoord p1, cCoord p2, cCoord p3) { _p1 = p1; _p2 = p2; _p3 = p3; }
 void getPoints(cCoord* p1, cCoord* p2, cCoord* p3) { *p1 = _p1; *p2 = _p2; *p3 = _p3; }

 void draw(void);

};

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cLine : public cShape
{
private:
 cCoord _p1, _p2;

public:
 cLine(cCoord p1, cCoord p2) : cShape(), _p1(p1), _p2(p2) {}

 void setPoints(cCoord p1, cCoord p2) { _p1 = p1; _p2 = p2; }
 void getPoints(cCoord* p1, cCoord* p2) { *p1 = _p1; *p2 = _p2; }

 void draw(void);

};

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cCube : public cShape
{
private:
 double _dimension;

 bool _solid;

public:
 cCube(double d) : cShape(), _dimension(d), _solid(true) {}

 void dimensions(double d) { _dimension = d; }
 double dimensions(void) { return _dimension; }

 void solid(bool b) { _solid = b; }
 bool solid(void) { return _solid; }

 void draw(void);

};

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cSphere : public cShape
{
private:
 double _radius;
 double _slices;
 double _stacks;

 bool _solid;

public:
 cSphere(double rd, double sl, double st) : cShape(), _radius(rd), _slices(sl), _stacks(st), _solid(true) {}

 void radius(float rd) { _radius = rd; }
 double radius(void) { return _radius; }

 void slices(float sl) { _slices = sl; }
 double slices(void) { return _slices; }

 void stacks(float st) { _stacks = st; }
 double stacks(void) { return _stacks; }

 void solid(bool b) { _solid = b; }
 bool solid(void) { return _solid; }

 void draw(void);

};

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
class cRect : public cShape
{
private:
 float _dx, _dy, _dz;

public:
 cRect(float dx, float dy, float dz) : cShape(), _dx(dx/2.0), _dy(dy/2.0), _dz(dz/2.0) {}
 ~cRect() {}

 void dimensions(float dx, float dy, float dz) { _dx = dx/2.0; _dy = dy/2.0; _dz = dz/2.0; }
 void dimensions(float* dx, float* dy, float* dz) { *dx = 2.0*_dx; *dy = 2.0*_dy; *dz = 2.0*_dz; }

 void dx(float v) { _dx = v/2.0; }
 float dx(void) { return 2.0*_dx; }

 void dy(float v) { _dy = v/2.0; }
 float dy(void) { return 2.0*_dy; }

 void dz(float v) { _dz = v/2.0; }
 float dz(void) { return 2.0*_dz; }

 void draw(void);

 void drawTexture(void);
};

}

#endif
