// ------------------------------------------------------------------------
// SceneGraph Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/05/20 18:52:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#include "transform.h"

namespace SceneGraph
{

//------------------------------------------------------------
//------------------------------------------------------------
void cTransform::LoadIdentity(void)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glLoadIdentity();
 glGetFloatv(GL_MODELVIEW_MATRIX, _matrix);
 glGetFloatv(GL_MODELVIEW_MATRIX, _imatrix);
 glPopMatrix();
}

//------------------------------------------------------------
//------------------------------------------------------------
void cTransform::Translate(float dx, float dy, float dz)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glLoadMatrixf(_matrix);
 glTranslatef(dx, dy, dz);
 glGetFloatv(GL_MODELVIEW_MATRIX, _matrix);
 glPopMatrix();

 glPushMatrix();
 glLoadMatrixf(_imatrix);
 glTranslatef(-dx, -dy, -dz);
 glGetFloatv(GL_MODELVIEW_MATRIX, _imatrix);
 glPopMatrix();
}

//------------------------------------------------------------
//------------------------------------------------------------
void cTransform::Rotate(float angle, float rx, float ry, float rz)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glLoadMatrixf(_matrix);
 glRotatef(angle, rx, ry, rz);
 glGetFloatv(GL_MODELVIEW_MATRIX, _matrix);
 glPopMatrix();

 glPushMatrix();
 glLoadMatrixf(_imatrix);
 glRotatef(-angle, rx, ry, rz);
 glGetFloatv(GL_MODELVIEW_MATRIX, _imatrix);
 glPopMatrix();
}

//------------------------------------------------------------
//------------------------------------------------------------
void cTransform::Scale(float ax, float ay, float az)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glLoadMatrixf(_matrix);
 glScalef(ax, ay, az);
 glGetFloatv(GL_MODELVIEW_MATRIX, _matrix);
 glPopMatrix();

 glPushMatrix();
 glLoadMatrixf(_imatrix);
 glScalef(1/ax, 1/ay, 1/az);
 glGetFloatv(GL_MODELVIEW_MATRIX, _imatrix);
 glPopMatrix();
}

//------------------------------------------------------------
//------------------------------------------------------------
int cTransform::SetupCamera(void)
{
 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();
 for(it = b; it != e; it++)
 {
  if(it->second->SetupCamera())
  {
   glMatrixMode(GL_MODELVIEW);
   glMultMatrixf(_imatrix);
   return 1;
  }
 }
 return 0;
}

//------------------------------------------------------------
//------------------------------------------------------------
int cTransform::SetupLights(void)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glMultMatrixf(_matrix);

 int n = 0;
 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();
 for(it = b; it != e; it++)
  n += it->second->SetupLights();

 glPopMatrix();

 return n;
}

//------------------------------------------------------------
//------------------------------------------------------------
void cTransform::Render(bool opaque)
{
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix();
 glMultMatrixf(_matrix);

 map<string, cNode*>::iterator it, b, e;
 b = _children.begin();
 e = _children.end();
 for(it = b; it != e; it++)
  it->second->Render(opaque);

 glPopMatrix();
}

}
